// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file ElInfo1d.h */

#ifndef AMDIS_ELINFO1D_H
#define AMDIS_ELINFO1D_H

#include "ElInfo.h"
#include "MemoryManager.h"

namespace AMDiS {

  /** \ingroup Traverse
   * \brief
   * ElInfo class for 1-dimensional elements (\ref Line).
   */
  class ElInfo1d : public ElInfo
  {
  public:
    MEMORY_MANAGED(ElInfo1d);

    /// Constructor. Calls ElInfo's protected Constructor.
    ElInfo1d(Mesh* aMesh) 
      : ElInfo(aMesh) 
    {}

    /// 1-dimensional realisation of ElInfo's fillElInfo method.
    void fillElInfo(int ichild, const ElInfo *elinfo_old);

    /// 1-dimensional realisation of ElInfo's fillMacroInfo method.
    void fillMacroInfo(const MacroElement*);

    /// 1-dimensional realisation of ElInfo's worldToCoord method.
    const int worldToCoord(const WorldVector<double>& w, DimVec<double>* l) const;

    /// 1-dimensional realisation of ElInfo's calcGrdLambda method.
    double calcGrdLambda(DimVec<WorldVector<double> >& grd_lam);

    /// 1-dimensional realisation of ElInfo's getNormal method.
    double getNormal(int side, WorldVector<double> &normal);

    /// 1-dimensional realisation of ElInfo's getElementNormal method.
    double getElementNormal( WorldVector<double> &normal) const;

    int getSideOfNeighbour(int i) { 
      return (i + 1) % 2; 
    }

    void getRefSimplexCoords(const BasisFunction *basisFcts,
			     DimMat<double> *coords) const;

    void getSubElementCoords(const BasisFunction *basisFcts,
			     int iChild,
			     DimMat<double> *coords) const;

  };

}

#endif // AMDIS_ELINFO1D_H
