// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file SolverMatrix.h */

#ifndef AMDIS_SOLVERMATRIX_H
#define AMDIS_SOLVERMATRIX_H


#include "DOFMatrix.h"


namespace AMDiS {

  // ============================================================================
  // ===== class SolverMatrix ===================================================
  // ============================================================================

  /**
   * \brief Helper class to provide complete matrix for ITL solvers and preconditioners
   */

  // General case undefined !
  template <typename Matrix>
  class SolverMatrix { };


  // DOFMatrix, i.e. ScalarProblem
  template <>
  class SolverMatrix<DOFMatrix>
  {
    public : 
      SolverMatrix() : matrix(0) {}

      void setMatrix(const DOFMatrix& A)
      {
	  matrix= &A.getBaseMatrix();
      }

      const DOFMatrix::base_matrix_type& getMatrix() const { return *matrix; }

    private:
      const DOFMatrix::base_matrix_type* matrix;
  };


  // VectorProblem
  template <>
  class SolverMatrix<Matrix<DOFMatrix*> >
  {
    public : 
      void setMatrix(const Matrix<DOFMatrix*>& A)
      {
	int ns = A.getSize(), size = A[0][0]->getFESpace()->getAdmin()->getUsedSize(),
	    new_size = ns * size;
	matrix.change_dim(new_size, new_size);
	DOFMatrix::inserter_type  ins(matrix);
	for (int rb= 0; rb < ns; ++rb)
	  for (int cb= 0; cb < ns; ++cb)
	    if (A[rb][cb]) 
	      ins[rb * size][cb * size] << A[rb][cb]->getBaseMatrix();	  
      }

      const DOFMatrix::base_matrix_type& getMatrix() const { return matrix; }

    private:      
       DOFMatrix::base_matrix_type matrix;
  };



} // namespace AMDiS

#endif // AMDIS_SOLVERMATRIX_H
